@@ -5,11 +5,13 @@ from typing import Dict, Tuple
 
 import pandas as pd
 from pytz import timezone
+from tqdm import tqdm
 
 from liualgotrader.common import config
 from liualgotrader.common.data_loader import DataLoader  # type: ignore
 from liualgotrader.common.database import fetch_as_dataframe
 from liualgotrader.common.tlog import tlog
+from liualgotrader.models.accounts import Accounts
 from liualgotrader.models.portfolio import Portfolio
 from liualgotrader.trading.trader_factory import trader_factory
 
@@ -104,6 +106,22 @@ def load_trades(day: date, end_date: date = None) -> pd.DataFrame:
     return loop.run_until_complete(fetch_as_dataframe(query))
 
 
+def load_client_trades(day: date, end_date: date = None) -> pd.DataFrame:
+    query = f"""
+    SELECT t.*, a.batch_id, a.algo_name
+    FROM 
+    new_trades as t, algo_run as a
+    WHERE 
+        t.algo_run_id = a.algo_run_id AND 
+        t.tstamp >= '{day}' AND 
+        t.tstamp < '{day + timedelta(days=1) if not end_date else end_date}' AND
+        t.expire_tstamp is null 
+    ORDER BY symbol, tstamp
+    """
+    loop = asyncio.get_event_loop()
+    return loop.run_until_complete(fetch_as_dataframe(query))
+
+
 async def aload_trades_by_batch_id(batch_id: str) -> pd.DataFrame:
     query = f"""
         SELECT 
@@ -127,11 +145,40 @@ async def aload_trades_by_batch_id(batch_id: str) -> pd.DataFrame:
     return df
 
 
+async def aload_trades_by_portfolio_id(portfolio_id: str) -> pd.DataFrame:
+    query = f"""
+        SELECT 
+            t.*, a.batch_id, a.start_time, a.algo_name
+        FROM 
+            new_trades as t, algo_run as a, portfolio_batch_ids as p
+        WHERE 
+            t.algo_run_id = a.algo_run_id AND 
+            a.batch_id = p.batch_id AND
+            p.portfolio_id = '{portfolio_id}' AND
+            t.expire_tstamp is null 
+        ORDER BY symbol, tstamp
+    """
+    df: pd.DataFrame = await fetch_as_dataframe(query)
+    try:
+        if not df.empty:
+            df["client_time"] = pd.to_datetime(df["client_time"])
+    except Exception:
+        tlog(
+            f"[Error] aload_trades_by_portfolio_id({portfolio_id}) can't convert 'client_time' column to datetime"
+        )
+    return df
+
+
 def load_trades_by_batch_id(batch_id: str) -> pd.DataFrame:
     loop = asyncio.get_event_loop()
     return loop.run_until_complete(aload_trades_by_batch_id(batch_id))
 
 
+def load_trades_by_portfolio(portfolio_id: str) -> pd.DataFrame:
+    loop = asyncio.get_event_loop()
+    return loop.run_until_complete(aload_trades_by_portfolio_id(portfolio_id))
+
+
 def load_runs(day: date, end_date: date = None) -> pd.DataFrame:
     query = f"""
      SELECT * 
@@ -325,53 +372,52 @@ def calc_symbol_trades_returns(
     daily_returns: pd.DataFrame,
     data_loader: DataLoader,
 ):
-    started = ended = None
-    qty: int
-
+    t1 = t2 = None
+    qty: int = 0
+    eastern = timezone("US/Eastern")
     for _, row in symbol_trades.iterrows():
-        if row.operation == "buy" and not started:
-            qty = row.qty
-            price = float(row.price)
-            started = daily_returns.index.get_loc(str(row.client_time.date()))
-            ended = None
-        elif row.operation == "sell" and started is not None:
-            ended = daily_returns.index.get_loc(str(row.client_time.date()))
-            for i in range(started, ended):
+        if t1 is None:
+            t1 = daily_returns.index.get_loc(str(row.client_time.date()))
+        else:
+            t2 = daily_returns.index.get_loc(str(row.client_time.date()))
+
+        if t1 is not None and t2 is not None:
+            for i in range(t1, t2):
                 value = (
                     qty
                     * data_loader[symbol].close[
-                        daily_returns.index[i]
-                        .to_pydatetime()
-                        .replace(tzinfo=est)
+                        eastern.localize(
+                            daily_returns.index[i].to_pydatetime()
+                        ).replace(hour=9, minute=30)
                     ]
                 )
-                daily_returns.loc[daily_returns.index[i], "agg_value"] += value
-                daily_returns.loc[daily_returns.index[i], "cash"] -= (
-                    qty * price
-                )
-            daily_returns.loc[daily_returns.index[ended], "cash"] += (
-                qty
-                * data_loader[symbol].close[
-                    daily_returns.index[ended]
-                    .to_pydatetime()
-                    .replace(tzinfo=est)
-                ]
-            )
+                daily_returns.loc[
+                    daily_returns.index[i],
+                    "equity",
+                ] += value
 
-            started = None
-            ended = None
+        if row.operation == "buy":
+            qty += row.qty
+        elif row.operation == "sell":
+            qty -= row.qty
 
-    if started and not ended:
-        ended = len(daily_returns.index)
-        for i in range(started, ended):
+        if qty == 0:
+            t1 = t2 = None
+        elif t2 is not None:
+            t1 = t2
+            t2 = None
+
+    if qty and t1:
+        for i in range(t1, len(daily_returns.index)):
             value = (
                 qty
                 * data_loader[symbol].close[
-                    daily_returns.index[i].to_pydatetime().replace(tzinfo=est)
+                    eastern.localize(
+                        daily_returns.index[i].to_pydatetime()
+                    ).replace(hour=9, minute=30)
                 ]
             )
-            daily_returns.loc[daily_returns.index[i], "agg_value"] += value
-            daily_returns.loc[daily_returns.index[i], "cash"] -= qty * price
+            daily_returns.loc[daily_returns.index[i], "equity"] += value
 
 
 def calc_batch_returns(batch_id: str) -> pd.DataFrame:
@@ -402,10 +448,10 @@ def calc_batch_returns(batch_id: str) -> pd.DataFrame:
     return pd.DataFrame(td, columns=["agg_value", "cash", "totals"])
 
 
-def compare_to_symbol_returns(batch_id: str, symbol: str) -> pd.DataFrame:
+def compare_to_symbol_returns(portfolio_id: str, symbol: str) -> pd.DataFrame:
 
     data_loader = DataLoader()
-    trades = load_trades_by_batch_id(batch_id)
+    trades = load_trades_by_portfolio(portfolio_id)
     start_date = trades.client_time.min().date()
     end_date = trades.client_time.max().date()
     trader = trader_factory()()
@@ -418,3 +464,46 @@ def compare_to_symbol_returns(batch_id: str, symbol: str) -> pd.DataFrame:
         axis=1,
     )
     return td[symbol]
+
+
+def get_cash(account_id: int, initial_cash: float) -> pd.DataFrame:
+    loop = asyncio.get_event_loop()
+    df = loop.run_until_complete(Accounts.get_transactions(account_id))
+    df = df.groupby(df.index.date).sum()
+    df.iloc[0].amount += initial_cash
+    r = pd.date_range(start=df.index.min(), end=df.index.max())
+    df.reindex(r).fillna(method="ffill")
+    df["cash"] = df.amount.cumsum()
+    df.index.rename("date", inplace=True)
+    return df
+
+
+def calc_portfolio_returns(portfolio_id: str) -> pd.DataFrame:
+    loop = asyncio.get_event_loop()
+    _ = loop.run_until_complete(Portfolio.load_by_portfolio_id(portfolio_id))
+
+    account_id, initial_account_size = loop.run_until_complete(
+        Portfolio.load_details(portfolio_id)
+    )
+    data_loader = DataLoader()
+    trades = load_trades_by_portfolio(portfolio_id)
+    start_date = trades.client_time.min().date()
+    end_date = trades.client_time.max().date()
+    trader = trader_factory()()
+
+    td = trader.get_trading_days(start_date=start_date, end_date=end_date)
+
+    td["equity"] = 0.0
+
+    cash_df = get_cash(account_id, initial_account_size)
+    symbols = trades.symbol.unique().tolist()
+    for i in tqdm(range(len(symbols))):
+        symbol = symbols[i]
+        symbol_trades = trades[trades.symbol == symbol].sort_values(
+            by="client_time"
+        )
+        calc_symbol_trades_returns(symbol, symbol_trades, td, data_loader)
+    td = td.join(cash_df)
+    td = td.fillna(method="ffill")
+    td["totals"] = td["equity"] + td["cash"]
+    return pd.DataFrame(td, columns=["equity", "cash", "totals"])
