@@ -16,7 +16,7 @@ latest_cost_basis: Dict[str, float] = {}
 latest_scalp_basis: Dict[str, float] = {}
 sell_indicators: Dict[str, Dict] = {}
 buy_indicators: Dict[str, Dict] = {}
-positions: Dict[str, int] = {}
+positions: Dict[str, float] = {}
 target_prices: Dict[str, float] = {}
 stop_prices: Dict[str, float] = {}
 partial_fills: Dict[str, int] = {}
