@@ -45,6 +45,13 @@ class NewTrade:
     ):
         async with pool.acquire() as con:
             async with con.transaction():
+                try:
+                    indicators_s = json.dumps(
+                        self.indicators or {}, allow_nan=False
+                    )
+                except ValueError:
+                    indicators_s = json.dumps({})
+
                 self.trade_id = await con.fetchval(
                     """
                         INSERT INTO new_trades (algo_run_id, symbol, operation, qty, price, indicators, client_time, stop_price, target_price)
@@ -56,7 +63,7 @@ class NewTrade:
                     self.operation,
                     self.qty,
                     self.price,
-                    json.dumps(self.indicators if self.indicators else {}),
+                    indicators_s,
                     client_buy_time,
                     stop_price,
                     target_price,
@@ -75,7 +82,7 @@ class NewTrade:
 
     @classmethod
     async def load_latest(
-        cls, pool: Pool, symbol: str, strategy_name: str, env: str
+        cls, pool: Pool, symbol: str, strategy_name: str
     ) -> Tuple[int, float, float, float, Dict, datetime]:
         async with pool.acquire() as con:
             async with con.transaction():
@@ -86,27 +93,24 @@ class NewTrade:
                         WHERE 
                             t.algo_run_id=a.algo_run_id AND
                             a.algo_name=$2 AND
-                            symbol=$1 AND
-                            algo_env=$3
+                            symbol=$1 
                         ORDER BY tstamp DESC LIMIT 1
                     """,
                     symbol,
                     strategy_name,
-                    env,
                 )
 
                 if row:
                     return (
                         int(row[0]),
                         float(row[1]),
-                        float(row[2]),
-                        float(row[3]),
-                        json.loads(row[4]),
+                        float(row[2] or 0.0),
+                        float(row[3] or 0.0),
+                        json.loads(row[4] or "{}"),
                         row[5],
                     )
-                else:
-                    tlog(f"{symbol} no data for strategy {strategy_name}")
-                    raise ValueError
+                tlog(f"{symbol} no data for strategy {strategy_name}")
+                raise ValueError
 
     @classmethod
     async def get_run_symbols(
@@ -154,3 +158,24 @@ class NewTrade:
                     old_run_id,
                     symbol,
                 )
+
+    @classmethod
+    async def get_latest_algo_run_id(
+        cls, symbol: str, pool: Pool = None
+    ) -> int:
+        if not pool:
+            pool = config.db_conn_pool
+        async with pool.acquire() as con:
+            async with con.transaction():
+                val = await con.fetchval(
+                    """
+                        SELECT  algo_run_id
+                        FROM new_trades
+                        WHERE symbol = $1
+                        ORDER BY tstamp DESC
+                        LIMIT 1
+                    """,
+                    symbol,
+                )
+
+                return val
